;**********************************************************************                                                                    *
;    Filename: Oscil.asm (Oscillograf->MT12864A)
;    Date: 18.04.03                                                           
;    Author:  Kichigin
;    Company: Tehno-Luch
;    Notes:
;**********************************************************************
	list      p=16f873         ; list directive to define processor
	radix	hex
	#include <p16f873.inc>     ; processor specific variable definitions
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _HS_OSC & _WRT_ENABLE_ON & _LVP_OFF & _DEBUG_OFF & _CPD_OFF 
;***** VARIABLE DEFINITIONS
w_temp        EQU     0x70        ; variable used for context saving 
status_temp   EQU     0x71        ; variable used for context saving
Count		equ	0x20
ColStolb	equ	0x21	;col. stolbcov in 1 controller - 64dec
ColPage		equ	0x22	;col. stranic - 8
ColSimb		equ 0x23	;col. simbol mult
nPage		equ	0x24	;number Page screen
Temp1		equ	0x25	;1razr. indic. (low)
Temp2		equ	0x26	;2
Temp3		equ	0x27	;3
Temp4		equ	0x28	;4 - - - - - - (high)
DecTx1		equ	0x29	;1razr. Tx(ADC),EEPROM, (low)
DecTx2		equ	0x2A	;2
DecTx3		equ	0x2B	;3
DecTx4		equ	0x2C	;4 - - - - - - - - - - (high)
DataADCl	equ	0x2D	;bin (low)
DataADCh	equ	0x2E	; -  (high)
DecTz1		equ	0x2F	;1 razr. (low)
DecTz2		equ	0x30	;2
DecTz3		equ	0x31	;3
DecTz4		equ	0x32	;4 - - - (high)
DataTzl		equ	0x33	;low razr.(bin)
DataTzh		equ	0x34	;high razr.(bin)
TimeX		equ	0x35	;counter time with TMR1
kX			equ	0x36	;koef.X (kX=0 -> A)
kXtmp		equ	0x37	;
kY			equ	0x38	;koef.Y (kY=x1, x2, x4)
Temp		equ	0x40
tmp1		equ	0x41
tmp2		equ	0x42
tmp3		equ	0x43
tmp4		equ	0x44
wrData		equ	0x45
;*************************************************************
		ORG     0x000             ; processor reset vector
		clrf    PCLATH            ; ensure page bits are cleared
  		goto    main              ; go to beginning of program
;----------------------------------------------------------------------
		ORG     0x004             ; interrupt vector location
		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register
		call	inTime				;count Timer
		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt
;xxxxxxxxxx tabl. simbol 0-9,A  (4x8bit) xxxxxxxxxxxxxxxxxx
TabSimb
		addwf	PCL,F
		retlw	B'01111110'		;simbol 0
		retlw	B'10000001'
		retlw	B'10000001'
		retlw	B'01111110'
		retlw	0				;1
		retlw	B'00000010'
		retlw	B'11111111'
		retlw	0
		retlw	B'11100010'		;2
		retlw	B'10010001'
		retlw	B'10001001'
		retlw	B'10000110'
		retlw	B'10001001'		;3
		retlw	B'10001101'
		retlw	B'10001011'
		retlw	B'01110001'
		retlw	B'00001111'		;4
		retlw	B'00001000'
		retlw	B'00001000'
		retlw	B'11111111'
		retlw	B'10001111'		;5
		retlw	B'10001001'
		retlw	B'10001001'
		retlw	B'01110001'
		retlw	B'01111110'		;6
		retlw	B'10001001'
		retlw	B'10001001'
		retlw	B'01110010'
		retlw	B'11100001'		;7
		retlw	B'00010001'
		retlw	B'00001001'
		retlw	B'00000111'
		retlw	B'01110110'		;8
		retlw	B'10001001'
		retlw	B'10001001'
		retlw	B'01110110'
		retlw	B'01001110'		;9
		retlw	B'10010001'
		retlw	B'10010001'
		retlw	B'01111110'
		retlw	B'11111000'		;A
		retlw	B'00010100'
		retlw	B'00010010'
		retlw	B'11111111'
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Tabtay
		addwf	PCL,F
		retlw	0x12		;tayPusk for kX=0, DataTzl
		retlw	0x01		;DataTzh,  T=5ms/screen
		retlw	0x01
		retlw	0x01
		retlw	0x90		;tayPusk for kX=1, DataTzl
		retlw	0x01		;DataTzh,  T=15ms/scr.
		retlw	0x01
		retlw	0x01
		retlw	0xE0		;tayPusk for kX=2, DataTzl
		retlw	0x03		;DataTzh,  T=50ms/scr.
		retlw	0x01
		retlw	0x01
		retlw	0xE0		;tayPusk for kX=3, TMR1L
		retlw	0xFE		;TMR1H
		retlw	0xFF		;TimeX
		retlw	0x01		;t=1.2ms/dot, T=150ms/screen
		retlw	0x39		;tayPusk for kX=4, TMR1L
		retlw	0xF7		;TMR1H
		retlw	0xFF		;TimeX
		retlw	0x01		;t=3.9ms/dot, T=500ms/screen
		retlw	0xA1		;tayPusk for kX=5, TMR1L
		retlw	0xE5		;TMR1H
		retlw	0xFF		;TimeX
		retlw	0x01		;t=11.72ms/dot, T=1500ms/scr.
		retlw	0x3F		;tayPusk for kX=6, TMR1L
		retlw	0xA8		;TMR1H
		retlw	0xFF		;TimeX
		retlw	0x01		;t=5s/dot, T=5s/screen
		retlw	0x58		;tayPusk for kX=7, TMR1L
		retlw	0xF8		;TMR1H
		retlw	0xFE		;TimeX
		retlw	0x01		;t=0.117s/dot, T=15s/scr.
		retlw	0x7E		;tayPusk for kX=8, TMR1L
		retlw	0x92		;TMR1H
		retlw	0xFC		;TimeX
		retlw	0x01		;t=0.39s/dot, T=50s/scr.
		retlw	0xFB		;tayPusk for kX=9, TMR1L
		retlw	0xB2		;TMR1H
		retlw	0xF5		;TimeX
		retlw	0x01		;t=1.172s/dot, T=150s/scr.
		retlw	0xFF		;tayPusk for kX=A, TMR1L
		retlw	0xB8		;TMR1H
		retlw	0xDD		;TimeX
		retlw	0x01		;t=3.9s/dot, T=500s/scr.

;***********************************************************************
main	bcf		STATUS,RP1		;bank 0,	Adjustment PORT
		bcf		STATUS,RP0		;
		clrf	PORTA
		clrf	PORTB
		clrf	PORTC
		movlw	B'10000001'		;32Tosc.,0-channel,ADC<-ON
		movwf	ADCON0
		bsf		STATUS,RP0		;bank 1
		movlw	B'00000100'		;Left ADFM, RA0,1,3-analog
		movwf	ADCON1			;
		movlw	B'00001011'		;RA0,1,3 < in, RA2,4,5 > out
		movwf	TRISA			;
		movlw	B'00011111'		;RB0-4 < in, RB5-7 > out
		movwf	TRISB			;
		movlw	0				;RC0-7 -> out (data LCD)
		movwf	TRISC			;
		movlw	B'00000111'		;osc+prDel+TMR0, Kpd=256,RB<podtRezist
		movwf	OPTION_REG		;Ft=18.6/4=4,65 Fpd=4,65/256=0,0181640625
		movlw	B'00000001'		;TMR1 interrupt ON
		movwf	PIE1
		bcf 	STATUS,RP0 		;bank 0
		movlw	B'00110000'		;TMR1 OFF <-Ftact/8=0,58125
		movwf	T1CON
		movlw	B'11000000'		;global and perif. interrupt ON
		movwf	INTCON
		bsf		PORTA,4			;Reset LCD OFF
		call	tay1
		call	Reset			;reset display
		call	tay2
		movlw	B'00000100'
		movwf	kX				;2channel
		movwf	kY				;x4, full screen
;********** main cycle ******************************************
start	
		btfss	PORTB,4			;btn Rejim
		goto	Mult			;Yes Multimetr
		btfss	PORTB,0			;No->Oscil, btn Start ?
		goto	Oscil			;Yes Start
		btfss	PORTB,1			;No, btn clrScreen ?
		call	clrScr			;Yes CLS
		btfss	PORTB,2			;No, btn kY ?
		call	setkY			;Yes
		btfss	PORTB,3			;No, btn kX ?
		call	setkX			;Yes
		call	tay3			;No
		call	tay3
		goto 	start			;wait btn Start
		
;xxxxxxxxxxxx Oscillograph xxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Oscil
		movf	kX,W
		sublw	0x02
		btfss	STATUS,C
		goto	Pusk2			;kX > 2, 2 channel
		goto	Pusk1			;kX=< 2, 1 channel
		
;xxxxxxxxx pusk TMR1 for 2 channel xxxxxxxxxxxxxxxxx
Pusk2
		bsf	T1CON,0		;TMR1 <- start
		movf	kX,W			;set X speed (TMR1<-Ftact/8)
		movwf	kXtmp
		bcf		STATUS,C
		rlf		kXtmp,F
		rlf		kXtmp,F
		movf	kXtmp,W
		call	Tabtay
		movwf	TMR1L			;low byte TMR1
		movf	kXtmp,W
		addlw	0x01
		call	Tabtay
		movwf	TMR1H			;high byte TMR1
		movf	kXtmp,W
		addlw	0x02
		call	Tabtay
		movwf	TimeX			;counter time
		bcf		PORTA,5			;E2<-0, crystal 1
		bsf		PORTA,2			;E1<-1
		bcf		ADCON0,3		;select channel 0(RA0)
		clrf	ColStolb		;N stolb=0
		movf	ColStolb,W		;set stolbec
		movwf	PORTC
		call	setColon
		goto 	start

;xxxxxxxxxxx Pusk 2 channel (1tact from TMR1) xxxxxxxxxxxxxxxxxxxxxxxx
inTime
		bcf		PIR1,0			;reset flag int. TMR1
		bcf		ADCON0,3		;select channel 0(RA0)
		incfsz	TimeX,F
		return					;TimeX>0,->next interrupt
		movf	kXtmp,W			;TimeX=0, set time TMR1
		call	Tabtay			;(TMRL,H;TimeX)
		movwf	TMR1L			;low byte TMR1
		movf	kXtmp,W
		addlw	0x01
		call	Tabtay
		movwf	TMR1H			;high byte TMR1
		movf	kXtmp,W
		addlw	0x02
		call	Tabtay
		movwf	TimeX			;counter time

		call	tay2			; -> Izmerenie ADC
		call	IzmerADC		;0 channel
		movf	DataADCh,W		;6 low bit
		movwf	Temp3			;
		movwf	tmp3			;
		
		bsf		ADCON0,3		;select channel 1(RA1)
		rrf		tmp3,F			;select N page 0 kan.
		rrf		tmp3,F			;
		rrf		tmp3,F			;
		movlw	B'00000111'		;
		andwf	tmp3,F			;
		
			call	tay2
		call	IzmerADC		;1 channel
		movf	DataADCh,W		;6 low bit
		movwf	Temp4			;
		movwf	tmp4			;
		bcf		ADCON0,3		;select channel 0(RA0)
		rrf		tmp4,F			;select N page 1 kan.
		rrf		tmp4,F			;
		rrf		tmp4,F			;
		movlw	B'00000111'		;
		andwf	tmp4,F			;
		
		movlw	B'00000001'		;screen bit
		movwf	tmp2
		movlw	B'00000111'
		andwf	Temp4,F		
sdvig1k		movf	Temp4,F
		btfsc	STATUS,Z
		goto	chan0			;DataADC=0
		decf	Temp4,F			;DataADC>0
		bcf		STATUS,C
		rlf		tmp2,F			;tmp2<-screen bite 1kan.
		goto	sdvig1k			;next bit
chan0	movf	tmp2,W
		movwf	Temp4			;Temp4<-screen bit 1kan.

		movlw	B'00000001'		;screen bit 0 kan.
		movwf	tmp2
		movlw	B'00000111'
		andwf	Temp3,F		
sdvig0k		movf	Temp3,F
		btfsc	STATUS,Z
		goto	setPagX			;DataADC=0
		decf	Temp3,F			;DataADC>0
		bcf		STATUS,C
		rlf		tmp2,F			;tmp1<-screen bite 0kan.
		goto	sdvig0k			;next bit
setPagX		movf	tmp2,W
		movwf	Temp3			;Temp3<-screen bit 0kan.		
			movf	tmp3,W		;Page0=Page1 ???
		subwf	tmp4,W
		btfsc	STATUS,Z
		goto	ravno			;Page0=Page1 !!!
		movf	tmp3,W			;Page0<>Page1, N stranicy 0kan
		movwf	PORTC			;
		call	setPage
			movf	Temp3,W		;WRITE bit 0kan to screen
		movwf	PORTC
		call	wrBit
		movf	ColStolb,W
		movwf	PORTC
		call	setColon
			movf	tmp4,W		;N stranicy 1kan
		movwf	PORTC			;		
		call	setPage
			movf	Temp4,W		;WRITE bit 1kan to screen
		movwf	PORTC			;
		call	wrBit		
testStolb	incf	ColStolb,F	;next stolb
		btfss	ColStolb,6
		return					;N stolb<64, next stolb
		btfss	PORTA,2			;N stolb=64, end crystal
		goto	endCr			;end 2 crystal
		bcf		PORTA,2			;E1<-0, end crystal 0
		bsf		PORTA,5			;E2<-1, crystal 1
		call	tay1
		clrf	ColStolb
		clrf	PORTC
		call	setColon		;set 0 stolbec
		return
ravno	movf	tmp3,W			;Page0=Page1, N stranicy 0kan
		movwf	PORTC			;
		call	setPage
			movf	Temp3,W		;Write bit 0+1 to screen
		iorwf	Temp4,W
		movwf	PORTC
		call	wrBit
		goto	testStolb
endCr
		bcf		T1CON,0			;TMR1 OFF
		clrf	PORTB
		bsf		PORTA,2			;E1<-1, screen ON
		call	tay3
		return
;xxxxxxxxxxx Reset display xxxxxxxxxxxxxxxxxxxxxxxxx
Reset	bsf		PORTA,2			;E1<-1
		bsf		PORTA,5			;E2<-1
		call	tay1
		bcf		PORTA,4			;impuls Reset		
		call	tay1
		bsf		PORTA,4			;end impuls
		call	tay1
	;--------- display ON -------------
		clrf	PORTB			;E<-0,A0<-0, rd/wr<-0
		nop
		movlw	B'00111111'		;data Display <- ON
		movwf	PORTC			;
		call	tay1
		bsf		PORTB,5			;E<-1, enable write
		call	tay1
		clrf	PORTB			;E<-0, end write
		clrf	PORTC
		call	tay2
		return
;xxxxxxxxxxxxxxx clear screen xxxxxxxxxxxxxxxxxxxxx
clrScr	clrf	PORTB
		movlw	0x40			;number stolbcov in 1 controller
		movwf	ColStolb		;40h -> 64dec
		clrf	ColPage			;col. stranic
	;---------------------
		bcf		PORTA,5			;E2<-0
		nop
		bsf		PORTA,2			;E1<-1, 1 crystal
		nop
	;---------------------
clrNextPg	
		movf	ColPage,W		;number page
		movwf	PORTC			;set page
		call	setPage
	;-------------------
		clrf	PORTC			;set stolb <- 0
		call	setColon
	;------------------
clrNext		clrf	PORTC			;data<-0
		call	wrBit
	;------------------
		decfsz	ColStolb,F
		goto	clrNext			;next stolb
		movlw	0x40			;64dec
		movwf	ColStolb
		incf	ColPage,F
		btfss	ColPage,3
		goto	clrNextPg		;ColPage<8, Next page		
		btfss	PORTA,2			;ColPage=8, end crystal
		goto	endScr			;E1=0
		bcf		PORTA,2			;E1=1, E1<-0
		bsf		PORTA,5			;E2<-1, 2 crystal ON
		clrf	ColPage
		goto	clrNextPg
endScr	bsf		PORTA,2			;E1<-1, screen ON
		clrf	PORTB
		call	tay3
		return
;xxxxxxxxxxxxxxx Save to RAM->Screen xxxxxxxxxxxxxxx
Pusk1	
		movf	kX,W			;set tay cykle ADC
		movwf	Temp1
		bcf		STATUS,C
		rlf		Temp1,F
		bcf		STATUS,C
		rlf		Temp1,F			;kX*4
		movf	Temp1,W
		call	Tabtay
		movwf	DataTzl
		movf	Temp1,W
		addlw	0x01
		call	Tabtay
		movwf	DataTzh
	;-------------- pusk ADC --------------------
		bcf		ADCON0,3		;select channel 0(RA0)
		bcf		ADCON0,4		;
		bcf	STATUS,IRP			;bank 0,1 (kocv. adr.)
		movlw	0xA0
		movwf	FSR				;start adr.RAM(bank 1)
nextRAM1	call	tayPusk
		call	IzmerADC
		movf	DataADCh,W
		movwf	INDF			;DataADC/4->RAM(bank 1)
		incfsz	FSR,F			;next adr RAM
		goto	nextRAM1		;adr<>0
	;------- select bank 0 ---------
		movlw	0x68			;adr=0,>bank 0
		movwf	FSR				;start adr.RAM(bank 0)
nextRAM0	call	tayPusk
		call	IzmerADC
		movf	DataADCh,W
		movwf	INDF			;DataADC/4->RAM(bank 0)
		incf	FSR,F			;next adr.RAM
		btfss	FSR,7
		goto	nextRAM0		;adr.<80h
	;------- RAM -> Screen ----------
		clrf	PORTB			;adr=80h, end RAM !
		clrf	PORTC
		bcf		PORTA,5			;E2<-0
		bsf		PORTA,2			;E1<-1, 1 crystal
		call	tay1
		
		movlw	0x40			;number stolbcov in 1 controller
		movwf	ColStolb		;40h -> 64dec
			clrf	PORTB			;write stolb LCD
		movlw	B'01000000'		;set stolb <- 0
		movwf	PORTC			;
		bsf		PORTB,5			;E<-1, enable write
		call	tay1
		clrf	PORTB			;E<-0
		
		bcf	STATUS,IRP			;bank 0,1 (kocv. adr.)
		movlw	0xA0
		movwf	FSR				;start adr.RAM(bank 1)
nextStolb1	movf	INDF,W
		movwf	DataADCl		;dataRAM->DataADCl
	;------- nomer stranicy --------
		movwf	DataADCh
		rrf		DataADCh,F
		rrf		DataADCh,F
		rrf		DataADCh,F		;nomer stranicy
		movlw	B'00000111'
		andwf	DataADCh,W		;select bit(2-0),number
		movwf	PORTC			;
		movlw	B'10111000'		;N page - 0
		iorwf	PORTC,F
		clrf	PORTB			;write
		nop
		bsf		PORTB,5			;E<-1, enable write
		call	tay1
		clrf	PORTB			;E<-0
	;-------- data -> screen --------------
		movlw	B'00000111'
		andwf	DataADCl,F		;data page
		movlw	B'00000001'
		movwf	tmp1
sdvig1	movf	DataADCl,F
		btfsc	STATUS,Z
		goto	writScr1		;DataADCl=0
		decf	DataADCl,F		;DataADCl>0
		bcf		STATUS,C
		rlf		tmp1,F
		goto	sdvig1
writScr1	movf	tmp1,W				
		movwf	PORTC			;
		nop
		clrf	PORTB
		call	tay1
		bsf		PORTB,7			;A<-1, write screen
		nop
		bsf		PORTB,5			;E<-1, enable wr.screen
		call	tay1
		bcf		PORTB,5			;E<-0
		call	tay1
		bcf		PORTB,7			;A<-0
		incf	FSR,F			;next RAM(bank 1)
		decfsz	ColStolb,F		;control end crystal
		goto	nextStolb1		;next stolb(bank 1)
		bcf		PORTA,2			;E1<-0, end 1 crystal
	;-------- RAM 1bank -> 2 crystal -----------
		bsf		PORTA,5			;E2<-1, 2 crystal
		call	tay1
nextStolb2	movf	INDF,W
		movwf	DataADCl		;dataRAM->DataADCl
	;------- nomer stranicy --------
		movwf	DataADCh
		rrf		DataADCh,F
		rrf		DataADCh,F
		rrf		DataADCh,F		;nomer stranicy
		movlw	B'00000111'
		andwf	DataADCh,W		;select bit(2-0),number
		movwf	PORTC			;
		movlw	B'10111000'		;N page - 0
		iorwf	PORTC,F
		clrf	PORTB			;write
		nop
		bsf		PORTB,5			;E<-1, enable write
		call	tay1
		clrf	PORTB			;E<-0
	;-------- data -> screen --------------
		movlw	B'00000111'
		andwf	DataADCl,F		;data page
		movlw	B'00000001'
		movwf	tmp1
sdvig2	movf	DataADCl,F
		btfsc	STATUS,Z
		goto	writScr2		;DataADCl=0
		decf	DataADCl,F		;DataADCl>0
		bcf		STATUS,C
		rlf		tmp1,F
		goto	sdvig2
writScr2	movf	tmp1,W				
		movwf	PORTC			;
		nop
		nop
		clrf	PORTB
		call	tay1
		bsf		PORTB,7			;A<-1, write screen
		nop
		bsf		PORTB,5			;E<-1, enable wr.screen
		call	tay1
		bcf		PORTB,5			;E<-0
		call	tay1
		decf	ColStolb,F	
		incfsz	FSR,F			;next RAM(bank 1)
		goto	nextStolb2		;next stolb(bank 1)
	;-------- next bank 0 ->crystal 2 -----------
		movlw	0x68			;FSR=0(adr=0),->bank 0
		movwf	FSR				;start adr.RAM(bank 0)
nextStolb3	movf	INDF,W
		movwf	DataADCl		;dataRAM->DataADCl
	;------- nomer stranicy --------
		movwf	DataADCh
		rrf		DataADCh,F
		rrf		DataADCh,F
		rrf		DataADCh,F		;nomer stranicy
		movlw	B'00000111'
		andwf	DataADCh,W		;select bit(2-0),number
		movwf	PORTC			;
		movlw	B'10111000'		;N page - 0
		iorwf	PORTC,F
		clrf	PORTB			;write
		nop
		bsf		PORTB,5			;E<-1, enable write
		call	tay1
		clrf	PORTB			;E<-0
	;-------- data -> screen --------------
		movlw	B'00000111'
		andwf	DataADCl,F		;data page
		movlw	B'00000001'
		movwf	tmp1
sdvig3	movf	DataADCl,F
		btfsc	STATUS,Z
		goto	writScr3		;DataADCl=0
		decf	DataADCl,F		;DataADCl>0
		bcf		STATUS,C
		rlf		tmp1,F
		goto	sdvig3
writScr3	movf	tmp1,W				
		movwf	PORTC			;
		nop
		nop
		clrf	PORTB
		call	tay1
		bsf		PORTB,7			;A<-1, write screen
		nop
		bsf		PORTB,5			;E<-1, enable wr.screen
		call	tay1
		bcf		PORTB,5			;E<-0
		call	tay1
		decf	ColStolb,F
		incf	FSR,F			;next RAM 0bank
		btfss	FSR,7
		goto	nextStolb3		;next stolb(bank 0)
		bsf		PORTA,2			;1 crystal-> screen
		clrf	PORTB
		call	tay3
		goto	start

				
;************** Izmerenie (DataADCh,l <-Tx) ******************
IzmerADC
		bsf		ADCON0,GO		;start ADC
		btfsc	ADCON0,GO
		goto	$-1				;not ready
		movf	ADRESH,W		;end work ADC (ready)
		movwf	DataADCh		;high byte Tx
		movf	kY,F
		btfsc	STATUS,Z
		goto	ADCx1			;kY=0, (Z=1)
		btfsc	kY,0			;kY>0, (Z=0)
		goto	ADCx2			;kY=1
		rrf		DataADCh,F		;kY>1(=2),select high 6 bit
ADCx2	rrf		DataADCh,F		;
ADCx1	comf	DataADCh,F		;invert		
		movlw	B'00111111'		;
		andwf	DataADCh,F		;high 6 bit		
		return
;***** Convert BIN-->DEC (DataADCh,l->DecTx1,2,3,4) **********
;	(Temp;Temp1,2,3;tmp1,2 - work registr.)
ConvBinDec	movf	DataADCl,W	;temp1,2 <- DataADCh,l
		movwf	tmp1			;
		movf	DataADCh,W		;
		movwf	tmp2			;
		bcf	STATUS,C
		movlw	0x10			;16d cykle
		movwf	Count			;
		clrf	Temp1		;right 1/2 byte-5dec.high digit
		clrf	Temp2		;4 and 3 dec. digit
		clrf	Temp3		;2 and 1 dec. low digit
loop16		rlf	tmp1,F		;low bin razr.
		rlf	tmp2,F			;high bin razr.
		rlf	Temp3,F
		rlf	Temp2,F
		rlf	Temp1,F
		decfsz	Count,F		;end 16 cykl ?
		goto	adjDEC		;not 16
;--- end BIN>DEC, swap (TempN -->DataTxN)----------
		movlw	0x0F		;mask low razr.
		andwf	Temp3,W		;select 1dec. low razr.
		movwf	DecTx1		;1-low dec. razr.
		movlw	0xF0		;2 - dec razr.
		andwf	Temp3,F		;
		swapf	Temp3,W		;
		movwf	DecTx2		;
		movlw	0x0F		;3 - dec razr.
		andwf	Temp2,W		;
		movwf	DecTx3		;
		movlw	0xF0		;4 - dec rasr.
		andwf	Temp2,F		;
		swapf	Temp2,W		;
		movwf	DecTx4		;		
		retlw	0
;------------------------------
adjDEC		movlw	Temp3	;adres Temp3 ->W
		movwf	FSR			;index.adres.FSR<adres
		call	adjBCD
		movlw	Temp2		;adr. Temp2->W
		movwf	FSR			;adr.Temp2->FSR
		call	adjBCD
		movlw	Temp1		;adr.Temp1->W
		movwf	FSR			;adr.Temp1->FSR
		call	adjBCD
		goto	loop16		;next cykle(from 16)
adjBCD		movlw	3
		addwf	0,W			;cosven.adr. W+TempN=W
		movwf	Temp
		btfsc	Temp,3		; > 7 ?
		movwf	0			; >7, W>TempN
		movlw	0x30		; < 7
		addwf	0,W			;cosven. adr. W+TempN=W
		movwf	Temp
		btfsc	Temp,7		; > 7 ?
		movwf	0			; > 7, W>TempN
		retlw	0
;******* Convert BCD->BIN (DecTz4,3,2,1->DataTzh,l)
;	(tmp2,1 - work register)
mpy10b		addwf	DataTzl,F
		btfsc	STATUS,C
		incf	DataTzh,F
mpy10a		bcf	STATUS,C
		rlf	DataTzl,W	; DataTzl x 2
		movwf	tmp1		
		rlf	DataTzh,W
		movwf	tmp2
		bcf	STATUS,C
		rlf	DataTzl,F	;DataTzl x 2
		rlf	DataTzh,F
		bcf	STATUS,C
		rlf	DataTzl,F	;DataTzl x 2
		rlf	DataTzh,F
		bcf	STATUS,C
		rlf	DataTzl,F	;DataTzl x 2
		rlf	DataTzh,F
		movf	tmp1,W
		addwf	DataTzl,F	;DataTzl=DataTzl+tmp1
		btfsc	STATUS,C	;Carry ?
		incf	DataTzh,F	;Yes Carry !
		movf	tmp2,W		;No Carry !
		addwf	DataTzh,F	;DataTzh=DataTzh+tmp2
		return
ConvBCD_BIN	clrf	DataTzh		;start convert
		movf	DecTz4,W
		movwf	DataTzl		;load high dec.digit(4razr)
		call	mpy10a		
		movf	DecTz3,W	;load 3 dec.digit
		call	mpy10b
		movf	DecTz2,W	;load 2 dec.digit
		call	mpy10b
		movf	DecTz1,W	;load 1 dec.digit (low razr.)
		addwf	DataTzl,F	;DataTzl=DataTzl+DecTz1
		btfsc	STATUS,C	;Carry ?
		incf	DataTzh,F	;Yes Carry !
		return			;No Carry ! end convert.

;********* Pouse - long t shim1 ***************************
tay1	movlw	0x01
		movwf	tmp1
looptay1	decf	tmp1,F
		nop
		btfss	STATUS,Z
		goto	looptay1
		return
;********* Pouse - long2 ***************************
tay2	movlw	H'35'
		movwf	tmp1
looptay2	movlw	H'11'
		movwf	tmp2
looptmp2	decf	tmp2,F
		btfss	STATUS,Z
		goto	looptmp2
		decf	tmp1,F
		btfss	STATUS,Z
		goto	looptay2
		return
;********* Pouse - long3  ***************************
tay3	movlw	0xFF
		movwf	tmp1
looptay3	movlw	0xFF
		movwf	tmp2
looptmp3	decf	tmp2,F
		btfss	STATUS,Z
		goto	looptmp3
		decf	tmp1,F
		btfss	STATUS,Z
		goto	looptay3
		return
;xxxxxxx time step pusk xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
tayPusk
		movf	DataTzh,W
		movwf	DecTz2
tayP2	movf	DataTzl,W
		movwf	DecTz1
tayP1	decfsz	DecTz1,F
		goto 	tayP1
		decfsz	DecTz2,F
		goto	tayP2
		return		
;xxxxxxxxxxxxxx Multimetr xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Mult
		btfss	PORTB,0			;btn R ?
		goto	izmR			;Yes btn R
		btfss	PORTB,1			;No, btn CLS ?
		call	clrScr			;Yes btn CLS
		btfss	PORTB,2			;No, btn C ?
		goto	izmC			;Yes btn C
		goto	izmV			;No btn			
;---------- izmerenie R ------------------------------
izmR
		bsf		STATUS,RP0		;bank 1
		bsf		ADCON1,7		;right ADC data
		bcf		STATUS,RP0		;bank 0
		bsf		ADCON0,3		;select channel 3(RA3)
		bsf		ADCON0,4		;
		call	tay3
		
		bsf		ADCON0,GO		;start ADC
		btfsc	ADCON0,GO
		goto	$-1				;not ready
		movf	ADRESH,W		;end work ADC (ready)
		movwf	DataADCh		;high byte
		bsf		STATUS,RP0		;bank 1
		movf	ADRESL,W		;low bite ADC
		bcf		STATUS,RP0		;bank 0
		movwf	DataADCl		;
		call	ConvBinDec
	;write DecTx(4-1) to screen(crystal 2, page 0)
		bcf		PORTA,2			;2crystal E1<-0
		bsf		PORTA,5			;E2 <-1
		clrf	PORTC		;set page 0
		call	setPage
		clrf	PORTC			;set Colon 0
		call	setColon
		movlw	4			;col. simbol <- 3(DecTx3-1)
		movwf	ColSimb
		bcf		STATUS,IRP		;set bank 1,0
		movlw	DecTx4			;kosv.adr. DecTx4
		movwf	FSR				;adr. DecTx4
izmR1	movf	INDF,W
		movwf	wrData			;wrData <- DecTx(4-1)
		call	wrSimb
		decf	FSR,F			;next adr. DecTx
		decfsz	ColSimb,F
		goto	izmR1			;next simbol
							;end simbol
		bsf		STATUS,RP0		;bank 1, restore ADC
		bcf		ADCON1,7		;left ADC data
		bcf		STATUS,RP0		;bank 0
		bcf		ADCON0,3		;select channel 0(RA0)
		bcf		ADCON0,4		;
		clrf	DataTzh		;tay wait next izmer
		movlw	0xFF
		movwf	DataTzl
		call	tayPusk
		goto	start
;--------- izmerenie C ------------------------------
izmC
		btfss	PORTB,2
		goto	izmC		;btn C<ON
		btfss	PORTB,2		;btn C>OFF
		goto	izmC		;btn C<ON
							;btnC>OFF
		bsf		STATUS,RP0		;bank 1
		bsf		ADCON1,7		;right ADC data
		bcf		STATUS,RP0		;bank 0
		bsf		ADCON0,3		;select channel 3(RA3)
		bsf		ADCON0,4		;
		call	tay1
		clrf	DataADCh
		clrf	DataADCl
izmC1	bsf		ADCON0,GO		;start ADC
		movlw	0x01
		movwf	DataTzh		;tay wait next izmer
		movlw	0x4F
		movwf	DataTzl
		call	tayPusk		
		btfsc	ADCON0,GO
		goto	$-1				;not ready
		bsf		STATUS,RP0		;OK !, bank 1
		movf	ADRESL,W		;low bite ADC
		bcf		STATUS,RP0		;bank 0
		sublw	0x40			;64dec
		btfss	STATUS,C
		goto	izmC2			;C=0, ADC>=64, end count
		movf	ADRESH,F		;C=1, ADC<64, count C
		btfss	STATUS,Z
		goto	izmC2			;ADC>64
		incfsz	DataADCl,F		;Z=1, ADRESH=0, count C
		goto	izmC1			;no carry
		incf	DataADCh,F		;carry
		btfss	DataADCh,2
		goto	izmC1			;DataADC<1024
izmC2	call	ConvBinDec		;DataADC=1024, overflow
	;write DecTx(4-1) to screen(crystal 2, page 0)
		bcf		PORTA,2			;2crystal E1<-0
		bsf		PORTA,5			;E2 <-1
		clrf	PORTC		;set page 0
		call	setPage
		clrf	PORTC			;set Colon 0
		call	setColon
		movlw	4			;col. simbol <- 3(DecTx3-1)
		movwf	ColSimb
		bcf		STATUS,IRP		;set bank 1,0
		movlw	DecTx4			;kosv.adr. DecTx4
		movwf	FSR				;adr. DecTx4
izmC3	movf	INDF,W
		movwf	wrData			;wrData <- DecTx(4-1)
		call	wrSimb
		decf	FSR,F			;next adr. DecTx
		decfsz	ColSimb,F
		goto	izmC3			;next simbol
							;end simbol	
		bsf		STATUS,RP0		;bank 1, restore ADC
		bcf		ADCON1,7		;left ADC data
		bcf		STATUS,RP0		;bank 0
		bcf		ADCON0,3		;select channel 0(RA0)
		bcf		ADCON0,4		;
		movlw	0xFF
		movwf	DataTzh		;tay wait next izmer
		movlw	0xFF
		movwf	DataTzl
		call	tayPusk		
		goto	start
;-------- izmerenie V -------------------------------
izmV
		bsf		STATUS,RP0		;bank 1
		bsf		ADCON1,7		;right ADC data
		bcf		STATUS,RP0		;bank 0
		bcf		ADCON0,3		;select channel 0(RA0)
		bcf		ADCON0,4		;
		call	tay3
		bsf		ADCON0,GO		;start ADC
		btfsc	ADCON0,GO
		goto	$-1				;not ready
		movf	ADRESH,W		;end work ADC (ready)
		movwf	DataADCh		;high byte
		bsf		STATUS,RP0		;bank 1
		movf	ADRESL,W		;low bite ADC
		bcf		STATUS,RP0		;bank 0
		movwf	DataADCl		;
		bcf		STATUS,C		;DataADC/16
		rrf		DataADCh,F
		rrf		DataADCl,F
		bcf		STATUS,C		;
		rrf		DataADCh,F
		rrf		DataADCl,F
		bcf		STATUS,C		;
		rrf		DataADCh,F
		rrf		DataADCl,F
		bcf		STATUS,C		;
		rrf		DataADCh,F
		rrf		DataADCl,F		
		call	ConvBinDec
	;write DecTx(4-1) to screen(crystal 1, page 0)
		bsf		PORTA,2			;1 crystal E1<-1
		bcf		PORTA,5			;E2 <-0
		clrf	PORTC		;set page 0
		call	setPage
		clrf	PORTC			;set Colon 0
		call	setColon
		movlw	4			;col. simbol <- 3(DecTx3-1)
		movwf	ColSimb
		bcf		STATUS,IRP		;set bank 1,0
		movlw	DecTx4			;kosv.adr. DecTx4
		movwf	FSR				;adr. DecTx4
izmV1	movf	INDF,W
		movwf	wrData			;wrData <- DecTx(4-1)
		call	wrSimb
		decf	FSR,F			;next adr. DecTx
		decfsz	ColSimb,F
		goto	izmV1			;next simbol
							;end simbol
		bsf		STATUS,RP0		;bank 1, restore ADC
		bcf		ADCON1,7		;left ADC data
		bcf		STATUS,RP0		;bank 0
		movlw	0xFF		;tay wait next izmer
		movwf	DataTzh	
		movlw	0xFF
		movwf	DataTzl
		call	tayPusk
		movlw	0xFF		;tay wait next izmer
		movwf	DataTzh	
		movlw	0xFF
		movwf	DataTzl
		call	tayPusk		
		goto	start
		
;xxxxxx write bit to screen (PORTC <- Data) xxxxxxxxxx
wrBit	
		clrf	PORTB
		call	tay1
		call	tay1
		bsf		PORTB,7			;A<-1, write screen
		call	tay1
		bsf		PORTB,5			;E<-1, enable wr.screen
		call	tay1
		call	tay1
		bcf		PORTB,5			;E<-0
		call	tay1
		clrf	PORTB			;set Stolb
		call	tay1
		return
;xxxxx write simbol to screen (wrData <- simbol, ColStolb)
wrSimb
		clrf	ColStolb
		bcf		STATUS,C		;wrData x 4
		rlf		wrData,F
		bcf		STATUS,C
		rlf		wrData,F
wrSimb1		movf	wrData,W		;calculate simbol bit
		addwf	ColStolb,W		;W<-(kX*4)+ColStolb
		call	TabSimb
		movwf	PORTC
		call	wrBit			;bit simbol -> Screen
		incf	ColStolb,F
		btfss	ColStolb,2
		goto	wrSimb1			;ColStolb<4, next stolb
		clrf	PORTC		;ColStolb=4, write space
		call	wrBit
		return
;xxxxx set Colonky (PORTC,(5-0) <- adres Colon.(0-63)) xxx
setColon
		bsf		PORTC,6			;set Colonky(stolbec)
		call	tay1
		bsf		PORTB,5			;E<-1, enable write
		call	tay1
		clrf	PORTB			;E<-0
		call	tay1
		return
;xxxxx set Page (PORTC,(2-0) <- Number Page (0-7))
setPage
		movlw	B'10111000'		;Set page
		iorwf	PORTC,F
		nop
		clrf	PORTB
		call	tay1
		bsf		PORTB,5			;E<-1, enable set
		call	tay1
		clrf	PORTB			;E<-0
		call	tay1
		return
;xxxxx Set kX +simb.Screen xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;kX=0   (T=5.12ms,  40mks/dot,  1chan)
;kX=1 	(T=15.36ms, 0.12ms/dot, 1chan)
;kX=2	(T=51.2ms,  0.4ms/dot,  1chan)
;kX=3	(T=153.6ms, 1.2ms/dot,  2chan)
;kX=4	(T=512ms,   4ms/dot,    2chan)
;kX=5	(T=1536ms,  12ms/dot,   2chan)
;kX=6	(T=5.12sec, 40ms/dot,   2chan)
;kX=7	(T=15.36sec,0.12sec,    2chan)
;kX=8	(T=51.2sec, 0.4sec,     2chan)
;kX=9	(T=153.6sec, 1.2sec   2chan)
;kX=10d	(T-512sec, 4sec,      2chan)
setkX
		bsf		PORTA,2			;1crystal E1<-1
		bcf		PORTA,5			;E2 <-0
		clrf	PORTC			;set Page 0
		call	setPage
		clrf	ColStolb
		incf	kX,F
		movf	kX,W
		sublw	0x0A
		btfss	STATUS,C
		clrf	kX				;kX>10dec, (C=0)
		movf	kX,W			;kX=<10dec, (C=1)
		movwf	wrData
		bcf		STATUS,C		;kX x 4
		rlf		wrData,F
		bcf		STATUS,C
		rlf		wrData,F
setkX1	movf	ColStolb,W		;set stolbec
		movwf	PORTC
		call	setColon
		movf	wrData,W		;calculate simbol bit
		addwf	ColStolb,W		;W<-(kX*4)+ColStolb
		call	TabSimb
		movwf	PORTC
		call	wrBit			;bit simbol -> Screen
		incf	ColStolb,F
		btfss	ColStolb,2
		goto	setkX1			;ColStolb<4, next stolb
		bsf		PORTA,5			;2crystal ON, E2<-1
		return
;xxxxxxxxxxxx set kY + simbol to screen xxxxxxxxxxxxxxxxxxx
; kY=0	(kADC x 1)
; kY=1	(kADC x 2)
; kY=2  (kADC x 4)
setkY
		bsf		PORTA,2			;1crystal E1<-1
		bcf		PORTA,5			;E2 <-0
		clrf	PORTC			;set Page 0
		call	setPage
		clrf	ColStolb
		incf	kY,F
		movf	kY,W
		sublw	0x02
		btfss	STATUS,C
		clrf	kY				;kY>2dec, (C=0)
		movf	kY,W			;kY=<2dec, (C=1)
		movwf	wrData
		bcf		STATUS,C		;kY x 4
		rlf		wrData,F
		bcf		STATUS,C
		rlf		wrData,F
setkY1	movf	ColStolb,W		;set stolbec
		movwf	PORTC
		call	setColon
		movf	wrData,W		;calculate simbol bit
		addwf	ColStolb,W		;W<-(kX*4)+ColStolb
		call	TabSimb
		movwf	PORTC
		call	wrBit			;bit simbol -> Screen
		incf	ColStolb,F
		btfss	ColStolb,2
		goto	setkY1			;ColStolb<4, next stolb
		bsf		PORTA,5			;2crystal ON, E2<-1
		return		
		
		

		
		END                       ; directive 'end of program'

